#!/usr/bin/python

#import PyEPL symbols into this namespace:
from pyepl.locals import *
from math import cos, sin
from string import atoi
import random
import scipy
import scipy.interpolate

def prepare(exp,config):
    """
    This function sets up the experiment. It will read store the 
    order of commands and where 
    each command's sound file is located.
    """

    # start with an empty list for the sessions
    sessionList = []
    
    # for each session...
    for sessionNum in xrange(config.numSessions):
        # get session specific configuration
        sessionconfig = config.sequence(sessionNum)
    
	# start with an empty list for sounds
	actionList = []
	
	# Get the dictionary structure storing the location of the sound files
	dictionary = config.soundDict
	# Get the list of commands
	commands = config.commands

	# for each command
	#for index in range(len(commands)):
        index = 0
        while (index < len(commands)):
            # if the ISI is being set
            if(commands[index] == '('):
		# Set the command to DELAY
		action = "DELAY"
                index = index+1
                isi_delay = 0
                isi_jitter = 0
		# Get the ISI Delay
                while (commands[index] != ','):
                    delay_char = commands[index]
                    isi_delay = 10*isi_delay + atoi(delay_char)
                    index=index+1
                index=index+1
		# Get the ISI Jitter
                while (commands[index] != ')'):
                    jitter_char = commands[index]
                    isi_jitter = 10*isi_jitter + atoi(jitter_char)
                    index=index+1
                index=index+1
	    else:
		# get the location of the sound file or Action
		action=dictionary[commands[index]]
		isi_delay=0
		isi_jitter=0
		index=index+1
                
            print "action=%s\tdelay=%d\tjitter=%d"%(action,isi_delay,isi_jitter)
	    # add it to the list
	    actionList.append((action,isi_delay,isi_jitter))
	
	# Save the list of sounds to each session
	sessionList.append(actionList)

    #save the information
    exp.saveState(None,sessionList=sessionList,sessionNum=0)
    

def run(exp,config):
    """
    This function will run through the list of commands.  
    Afterwards it will present a
    continuously moving target for the subject to track with their eyes. 
    """

    #get the state
    state = exp.restoreState()

    target = Text('*')

    # set up session...
    if state.sessionNum >= len(state.sessionList):
        # if all the sessions have been run, don't continue
        print "No more sessions!"
        return

    # set the session number (so PyEPL knows what directory to put the data in)
    exp.setSession(state.sessionNum)

    # get session specific configuration
    sessionconfig = config.sequence(state.sessionNum)

    # create tracks...
    video = VideoTrack("video")
    audio = AudioTrack("audio")
    keyboard = KeyTrack("keyboard")
    log = LogTrack("session")

    # create a PresentationClock to handle timing
    clock = PresentationClock()

    # log the start of the session
    log.logMessage('%s\t%d' % ('SESS_START',state.sessionNum),clock)

    # Display the start instructions
    video.clear("black")
    instruct(sessionconfig.instructions_start,clk=clock)

    # clear the screen
    video.clear("black")

    # display the trial number and wait for a key to be pressed:
    waitForAnyKey(clock, Text(sessionconfig.instructions_pressAnyKey))

    #clear the screen
    video.clear("black")

    # Get the size of the screen...
    (xres,yres) = video.getResolution()

    # ...and the center of the screen.
    xcent = xres/2
    ycent = yres/2

    # Use framerate to set speed of spline animation
    video.setMaximumFramerate(sessionconfig.framerate)

    # Set up the target points
    video.showProportional(target,0.5,0.5)
    video.showProportional(target,0.0,0.5)
    video.showProportional(target,0.5,0.0)
    video.showProportional(target,0.5,1.0)
    video.showProportional(target,1.0,0.5)
    video.updateScreen(clock)

    # log the grid presentation
    log.logMessage('%s\t%d' % ('GRID_PRESENT',state.sessionNum),clock)

    # wait before presenting the first command
    clock.delay(sessionconfig.initial_delay)

    # present each command
    for index in range(len(state.sessionList[state.sessionNum])):
        (command,isi_delay,isi_jitter) = state.sessionList[state.sessionNum][index]
        print "command=%s\tdelay=%d\tjitter=%d"%(command,isi_delay,isi_jitter)
	if(command == "DELAY"):
	    # If the command is to delay, then delay
	    clock.delay(isi_delay,isi_jitter)
	elif(command == sessionconfig.track):
	    # If the command says to display the spline
	    xs = []
	    ys = []
	    
	    # Set the start points to the center of the screen
	    xpos = xcent
	    ypos = ycent
	    xs = [xpos]
	    ys = [ypos]
	    theta = sessionconfig.mean

	    for i in range(0,sessionconfig.num_points):
		# make sure the new points are on the screen
		while 1:
		    deltTheta = random.gauss(0,sessionconfig.st_dev)
		    theta = theta + deltTheta
		    deltX = sessionconfig.r_const * cos(theta)
		    deltY = sessionconfig.r_const * sin(theta)
		    xpos = xpos + deltX
		    ypos = ypos + deltY
		    # If the new points are on the screen
		    if( xpos>0+sessionconfig.border and 
			xpos<xres-sessionconfig.border and 
			ypos>0+sessionconfig.border and 
			ypos<yres-sessionconfig.border):
			# Then continue
			break
                    print "x = %d\ty = %d"%(xpos,ypos)
		    # Else, reset and try again
		    xpos = xpos - deltX
		    ypos = ypos - deltY
		    theta = theta - deltTheta

		xs = xs + [xpos]
		ys = ys + [ypos]
		print "xres= %d\tyres=%d"%(xres,yres)#"x = %d\ty = %d"%(xpos,ypos)

	    # Get the spline
	    xArr = scipy.array(xs)
	    yArr = scipy.array(ys)
	    tck,u = scipy.interpolate.splprep([xArr,yArr],s=0)
	    unew = scipy.arange(0,xres,sessionconfig.speed_param2)
	    xplt,yplt = scipy.interpolate.splev(unew,tck)

	    # Present spline instructions.
	    video.clear("black")
	    #waitForAnyKey(clock,Text(sessionconfig.instructions_spline))

	    # Log the start of the Spline
	    log.logMessage('%s\t%d' % ('SPLINE_START',state.sessionNum),clock)

	    # Show the spline
	    for index in range(0,len(xplt)):
		video.clear("black")
		if(index % sessionconfig.video_log_accuracy == 0):
		    log.logMessage('%s\t%s\t%d\t%s\t%d' % 
				   ('VIDEO_LOG',"X_COORD",xplt[index]
				    ,"Y_COORD",yplt[index]),clock)
		
		#for i in range(0,sessionconfig.num_points):
		#    video.show(Text("%d"%i,size=0.025),xs[i],ys[i])
		
		video.show(target,xplt[index],yplt[index])
		video.updateScreen(clock)
		if(xplt[index]<0 or xplt[index]>xres or yplt[index]<0 
		   or yplt[index]>yres):
		    break
	    print "x = %d\ty = %d"%(xplt[index],yplt[index])
	    # Set the clock to the current time
	    clock.tare()	
	else:
	    # Else, make sure the targets are shown ...
	    video.clear("black")
	    video.showProportional(target,0.5,0.5)
	    video.showProportional(target,0.0,0.5)
	    video.showProportional(target,0.5,0.0)
	    video.showProportional(target,0.5,1.0)
	    video.showProportional(target,1.0,0.5)
	    video.updateScreen(clock)
	    # ...and Play the sound file.
	    soundFile = FileAudioClip(command)
	    stamp = soundFile.present(clk=clock,duration=isi_delay,
				      jitter=isi_jitter)
	    clock.wait()
	    # log the sound presentation
	    log.logMessage('%s\t%s' % ('SOUND',
					   state.sessionList[state.sessionNum]
					   [index]),stamp)

    # End of session
    video.clear("black")
    waitForAnyKey(clock, Text(sessionconfig.instructions_finished))

    # log the end of the session
    log.logMessage('%s\t%d' % ('SESS_END',state.sessionNum),clock)

    # Save the state when the session is finished
    # with trial and session numbers set for the next session
    exp.saveState(state, sessionNum = state.sessionNum + 1)

    # wait for the clock to catch up 
    clock.wait()



# only do this if the experiment is run as a stand-alone program 
#(not imported as a library)...
if __name__ == "__main__":
    checkVersion('1.0.6')
    # set up the experiment...
    exp = Experiment()
    exp.parseArgs()
    exp.setup()

    # allow users to break out of the experiment with escape-F1 
    # (the default key combo)
    exp.setBreak()

    # get the subject configuration
    config = exp.getConfig()

    # if there was no saved state, run the prepare function
    if not exp.restoreState():
        prepare(exp, config)

    # now run the subject
    run(exp, config)
